assume cs:_TEXT
public _cpu_speed
_TEXT segment para public 'CODE'

comment~
************************************************************************
Function:		unsigned long _cpu_speed( unsigned int )
				"C" style caller
Purpose:		calculate elapsed time req’d to complete a loop of IDIVs
Technique:		Use the PC’s high resolution timer/counter chip (8254)
				to measure elapsed time of a software loop consisting
				of the IDIV and LOOP instruction.
Definitions:	The 8254 receives a 1.19318MHz clock (0.8380966 usec).
				One "tick" is equal to one rising clock edge applied
				to the 8254 clock input.
Inputs:			ax = no. of loops for cpu_speed_loop
Returns:		ax = no. of 1.19318MHz clk ticks req’d to complete a loop
				dx = state of 8254 out pin
***********************************************************************~
PortB			EQU		061h
Timer_Ctrl_Reg 	EQU		043h
Timer_2_Data	EQU		042h
stk$dx			EQU		10		;dx register offset
stk$ax			EQU		14		;dx register offset
stack$ax		EQU		[bp]+stk$ax
stack$dx		EQU		[bp]+stk$dx
Loop_Count		EQU		[bp+16]+4	

.386p


_cpu_speed		proc near
		pushf				;save interrupt flag
		pusha				;pushes 16 bytes on stack
		mov		bp,sp		;init base ptr

		cli					;disable interrupts

		
;-------disable clock to timer/counter 2
		in		al, PortB
		and		al, 0feh
		out		80h,al		;I/O recovery time
		out		PortB, al
		mov		di, ax
;-------initialize the 8254 counter to "0", known value
		mov		al,0b0h
		out		Timer_Ctrl_Reg, al		; control word to set channel 2 count
		out		80h,al					;I/O recovery time
		mov 	al,0ffh
		out 	Timer_2_Data,al			;init count to 0, lsb
		out		80h,al 					;I/O recovery time
		out 	Timer_2_Data,al			;init count to 0, lsb
		
;-------get the number of loops from the caller’s stack
		mov		cx,Loop_Count			;loop count

;-------load dividend & divisor, clk count for IDIV depend on operands!
		xor		edx,edx					;dividend EDX:EAX
		xor		eax,eax
		mov		ebx,1					;divisor

;-------enable the timer/counter’s clock. Begin timed portion of test!
		xchg	ax, di					;save ax for moment
		or		al, 1
		out		PortB, al				;enable timer/counter 2 clk
		xchg	ax, di					;restore ax

;-------this is the core loop.
		ALIGN 16
cpu_speed_loop:
		idiv		ebx
		idiv		ebx
		idiv		ebx
		idiv		ebx
		idiv		ebx
		loop		cpu_speed_loop
		
;-------disable	the timer/counter’s clk. End timed portion of test!
		mov		ax, di
		and		al, 0FEH
		out		PortB, al

;-------send latch status command to the timer/counter
		mov		 al, 0c8h			;latch status and count
		out		Timer_Ctrl_Reg, al
		out		80h,al				;I/O recovery time

;-------read status byte, and count value "ticks" from the timer/cntr
		in		al, Timer_2_Data	;read status
		out		80h,al				;I/O recovery time
		mov		dl, al
		and		dx, 080h
		shr		dx, 7
		in		al, Timer_2_Data	;read LSB
		out		80h,al				;I/O recovery time
		in		al, Timer_2_Data
		mov		bl, al
		in		al, Timer_2_Data	;read MSB
		out		80h,al				;I/O Recovery time
		mov		bh, al
		
		not		bx					;invert count
		
;-------send command to clear the timer/counter
		mov		al, 0b6h
		out		Timer_Ctrl_Reg, al	;clear channel 2 count
		out		80h,al				;I/O recovery time
		xor		al, al
		out		Timer_2_Data, al	;set count to 0, lsb
		out		80h,al				;I/O recovery time
		out		Timer_2_Data, al	;set count to 0, msb
		
;-------put return values on the stack for the caller
		mov		[bp+stk$ax], bx
		mov		[bp+stk$dx], dx
		popa
		popf	;restores interrupt flag
		ret
_cpu_speed		endp

.8086
_TEXT	ENDS
END
