/*
**********************************************************************
function:	 	main()									WCP 8/22/95
Purpose:		a driver program to demonstrate:
					CPU detection
					CPU core frequency in Mhz.
Returns:		 0 if successful

Required source code modules
	m1_stat.c	main() module (this file)
	id.asm		cpu identification code
	clock.asm	cpu timing loop

Compile and Link instructions for Borland C++ or equivalent:
	bcc m1_stat.c id.asm clock.asm
***********************************************************************
*/
/* include directives */
#include <stdio.h>
/* constants */
#define TTPS			1193182
 //high speed Timer Ticks per second in Mhz
#define MHZ				1000000
 //number of clocks in 1 Mhz
#define LOOP_COUNT		0x2000
 //core loop iterations
#define RUNS			10
 //number of runs to average
#define DIVS			5
 //# of IDIV instructions in the core loop
#define MII_IDIV_CLKS 	17
 //known clock counts for MII
#define MII_LOOP_CLKS 	1
#define P54_IDIV_CLKS 	46
 //known clock counts for P54
#define P54_LOOP_CLKS 	7

/* prototypes */
unsigned int iscyrix( void ); 				//detects cyrix cpu
unsigned long cpu_speed( unsigned int );	//core timing loop


int main(){
	/* declarations */
unsigned char uc_cyrix_cpu = 0; 	//Cyrix cpu? 0=no,1=yes
unsigned int i_runs = 0; 			//number of runs to avg
unsigned int ui_idiv, ui_loop = 0; 	//instruction clk counts
unsigned long ul_tt_cnt, ul_tt_sum = 0; //timer tick counts, sum
unsigned int ui_core_loop_cntr = LOOP_COUNT; //core loop iterations
float f_mtt = 0; //measured timer ticks
float f_total_core_clks = 0; //calculated core clocks
float f_total_time = 0; //measured mhz
float f_mhz = 0; //measured mhz


/* ********** determine if Cyrix CPU is present ************** */
//detect if Cyrix CPU is present
uc_cyrix_cpu = iscyrix(); //1=cyrix, 0=non-cyrix

//display a msg
if(uc_cyrix_cpu) printf("\nCyrix CPU present! ");
else printf("\nCyrix CPU not present! ");
/* ******************** determine CPU Mhz ******************** */
//count # of hi speed "timer ticks" to complete several runs of core loop
for (i_runs = 0 ; i_runs < RUNS ; i_runs++) {
	ul_tt_cnt = cpu_speed( ui_core_loop_cntr );
	ul_tt_sum += ul_tt_cnt;  //sum them all together
} //end for
//compute the avg number of high speed "timer ticks" for the several runs

f_mtt = ul_tt_sum / RUNS;  //compute the average
//initialize variables with the "known" clock counts for a MII or P54

if(uc_cyrix_cpu)ui_idiv=MII_IDIV_CLKS; else ui_idiv=P54_IDIV_CLKS;
if(uc_cyrix_cpu)ui_loop=MII_LOOP_CLKS; else ui_loop=P54_LOOP_CLKS;

//determine the total number of core clocks. (5 idivs are in the core loop)
f_total_core_clks = (float)ui_core_loop_cntr * (ui_idiv * DIVS + ui_loop);

//the time it took to complete the core loop can be determined by the
//ratio of measured timer ticks(mtt) to timer ticks per second(TTPS).
f_total_time = f_mtt / TTPS;

//frequency can be found by the ratio of core clks to the total time.
f_mhz = f_total_core_clks / f_total_time;
f_mhz = f_mhz / MHZ;  //convert to Mhz

//display a msg
printf("The core clock frequency is: %3.1f MHz\n\n",f_mhz);
return(0);

} //end main
